// ===========================================================================
// CONTENT  : TEST CLASS {ClassName}
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.0 - 19/12/2008
// HISTORY  :
//  19/12/2008  mdu  CREATED
//
// Copyright (c) 2008, by MDCS. All rights reserved.
// ===========================================================================
package org.pfsw.extender ;

// ===========================================================================
// IMPORTS
// ===========================================================================
import java.lang.reflect.Proxy;
import java.math.BigDecimal;

import org.pfsw.extender.testhelper.SimpleCalculator;
import org.pfsw.extender.testhelper.ISimpleCalculator;

import junit.framework.* ;

/**
 * Test class for corresponding business class.
 *
 * @author Manfred Duchrow
 * @version 1.0
 */
public class DelegatingInvocationHandlerTest extends TestCase
{
  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
  /**
   * Initialize the new instance with default values.
   */
  public DelegatingInvocationHandlerTest( String name )
  {
    super( name ) ;
  } // DelegatingInvocationHandlerTest() 

  // =========================================================================
  // PUBLIC CLASS METHODS
  // =========================================================================
  public static Test suite()
  {
		return new TestSuite( DelegatingInvocationHandlerTest.class ) ;
	} // suite() 

  // =========================================================================
  // TEST METHODS
  // =========================================================================

  public void test_null() 
	{
  	ISimpleCalculator calc = null;
  	
  	try
		{
  		calc = this.createProxyObject( ISimpleCalculator.class, null ) ;
			fail( "Must have thrown exception" ) ; 
		}
		catch ( RuntimeException e )
		{
			// Fine, works as expected
		}
		assertNull(calc);
	} // test_null()
	
	// -------------------------------------------------------------------------
  
  public void test_delegation()
  {
  	ISimpleCalculator calc ;
  	
  	calc = this.createProxyObject( ISimpleCalculator.class, new SimpleCalculator() ) ;
  	assertEquals( BigDecimal.valueOf(0), calc.getCurrentValue() ) ;
  	calc.add( new BigDecimal("22.15") ) ;
  	calc.add( new BigDecimal("3.35") ) ;
  	assertEquals( new BigDecimal("25.50"), calc.getCurrentValue() ) ;
  	calc.subtract( new BigDecimal("7.10") ) ;
  	calc.reset();
  	assertEquals( BigDecimal.valueOf(0), calc.getCurrentValue() ) ;
  } // test_delegation() 

  // -------------------------------------------------------------------------

  // =========================================================================
  // PROTECTED INSTANCE METHODS
  // =========================================================================
  @Override
  protected void setUp() throws Exception
  {
    super.setUp() ;
	} // setUp() 

  // -------------------------------------------------------------------------

	protected <I> I createProxyObject( Class<I> interfaceType, Object object ) 
	{
		DelegatingInvocationHandler invocationHandler ;
		
		invocationHandler = new DelegatingInvocationHandler(object) ; 
		return (I)Proxy.newProxyInstance( interfaceType.getClassLoader(), 
				new Class[] { interfaceType }, invocationHandler ) ;
	} // createProxyObject() 
	
	// -------------------------------------------------------------------------
	
} // class DelegatingInvocationHandlerTest 
