// ===========================================================================
// CONTENT  : TEST CLASS {ClassName}
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.0 - 19/12/2008
// HISTORY  :
//  19/12/2008  mdu  CREATED
//
// Copyright (c) 2008, by MDCS. All rights reserved.
// ===========================================================================
package org.pfsw.extender;

// ===========================================================================
// IMPORTS
// ===========================================================================
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.util.Locale;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.pf.reflect.ReflectUtil;
import org.pfsw.extender.testhelper.Division;
import org.pfsw.extender.testhelper.IBasicCalculator;
import org.pfsw.extender.testhelper.IBasicUnitCalculator;
import org.pfsw.extender.testhelper.IDivision;
import org.pfsw.extender.testhelper.IMultiplication;
import org.pfsw.extender.testhelper.IResultPrinter;
import org.pfsw.extender.testhelper.IUnit;
import org.pfsw.extender.testhelper.Multiplication;
import org.pfsw.extender.testhelper.SimpleCalculator;
import org.pfsw.extender.testhelper.UnitResultPrinter;

/**
 * Test class for corresponding business class.
 *
 * @author Manfred Duchrow
 * @version 1.0
 */
public class ObjectExtenderTest extends TestCase
{
	// =========================================================================
	// CONSTANTS
	// =========================================================================
	private static final ReflectUtil RU = ReflectUtil.current();

	// =========================================================================
	// INSTANCE VARIABLES
	// =========================================================================

	// =========================================================================
	// CONSTRUCTORS
	// =========================================================================
	public ObjectExtenderTest(String name)
	{
		super(name);
	} // ObjectExtenderTest() 

	// =========================================================================
	// PUBLIC CLASS METHODS
	// =========================================================================
	public static Test suite()
	{
		return new TestSuite(ObjectExtenderTest.class);
	} // suite() 

	// =========================================================================
	// TEST METHODS
	// =========================================================================

	public void test_functionality_extension_1()
	{
		IBasicCalculator calc;

		calc = ObjectExtender.createExtensibleObject(IBasicCalculator.class, new SimpleCalculator());
		ObjectExtender.setMixinTo(calc, IMultiplication.class, new Multiplication());
		ObjectExtender.setMixinTo(calc, IDivision.class, new Division());
		calc.setCurrentValue(new BigDecimal("9.00"));
		calc.multiply(new BigDecimal("2"));
		calc.add(new BigDecimal("0.75"));
		assertEquals(new BigDecimal("18.75"), calc.getCurrentValue());
		calc.divide(new BigDecimal("3"));
		assertEquals(new BigDecimal("6.25"), calc.getCurrentValue());
	} // test_functionality_extension_1() 

	// -------------------------------------------------------------------------

	public void test_functionality_extension_2()
	{
		IBasicUnitCalculator calc;
		UnitResultPrinter urp = new UnitResultPrinter();

		calc = ObjectExtender.createExtensibleObject(IBasicUnitCalculator.class, new SimpleCalculator());
		ObjectExtender.setMixinTo(calc, IMultiplication.class, new Multiplication());
		ObjectExtender.setMixinTo(calc, IDivision.class, new Division());
		ObjectExtender.setMixinTo(calc, IUnit.class, urp);
		ObjectExtender.setMixinTo(calc, IResultPrinter.class, urp);
		calc.setCurrentValue(new BigDecimal("12.40"));
		calc.divide(new BigDecimal("4"));
		calc.add(new BigDecimal("0.40"));
		calc.multiply(new BigDecimal("3"));
		assertEquals("10.50", calc.getResult());
		calc.setUnit("EUR");
		assertEquals("10.50 EUR", calc.getResult());
	} // test_functionality_extension_2() 

	// -------------------------------------------------------------------------

	public void test_isSameMethod_1()
	{
		Method m1;
		Method m2;
		ExtendedObject extended = new ExtendedObject(new Object());

		m1 = RU.getMethod(new Object(), "equals", new Class[] { Object.class });
		m2 = RU.getMethod(Locale.US, "equals", new Class[] { Object.class });
		assertNotNull(m1);
		assertNotNull(m2);
		assertFalse(m1.equals(m2));
		assertTrue(extended.isSameMethod(m1, m2));
	} // test_isSameMethod_1() 

	// -------------------------------------------------------------------------

	// =========================================================================
	// PROTECTED INSTANCE METHODS
	// =========================================================================
	@Override
  protected void setUp() throws Exception
	{
		super.setUp();
	} // setUp() 

	// -------------------------------------------------------------------------

} // class ObjectExtenderTest 
